/*
 *
 * Contents:
 *
 *   Setup()        - Prepare the printer for printing.
 *   StartPage()    - Start a page of graphics.
 *   EndPage()      - Finish a page of graphics.
 *   Shutdown()     - Shutdown the printer.
 *   CompressData() - Compress a line of graphics.
 *   main()         - Main entry and processing of driver.
 */

#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include "evolis.h"
#include "evoliserror.h"
#include "cups.h"
#include "ppd.h"
#include "raster.h"

#define LVERSION "3.02 RC12"

#define EVOLIS_1  0
#define CUPS15    1

#define PORTRAIT    1
#define LANDSCAPE   2 

#define MAX_MAG_DATA 256
#define MAX_PASSTHROUGH_SZ 2048

// ----------------------------------------------------
// Typedefs
// ----------------------------------------------------

typedef struct TrackInfo_tag {
    int type;                   // MAGTYPE_ISO1, MAGTYPE_ISO2, MAGTYPE_ISO3, MAGTYPE_SIPASS, MAGTYPE_C2_75, MAGTYPE_JIS2, MAGTYPE_C4, MAGTYPE_C2_210
    int density;                // 0:auto / 75 / 210
    char data[MAX_MAG_DATA];
} TrackInfo_t;

typedef struct MagInfo_tag {
    TrackInfo_t Track[3];
    int KeepCard;               // 1 true 0 false
    char Coercivity[5];         // HICO, LOCO, JIS, NONE
} MagInfo_t; 

typedef struct LamOption_tag {
    char    defLam;                 /* Param 1 de PLm  : 0, r, v, b     */
    char    exitLam;                /* Param 2 de PLm  : 0, r, v        */              
    int     lamType;                /* Param 2 de PLds : 0, 1, 2        */              
    int     lamSpeed;               /* Param 3 de PLds : 2-18           */
    int     lamTemp;                /* Param 4 de PLds : 100-180        */
    int     lamAlt;                 /* Param 5 de PLds : 0,1,2          */      
    char    lamDesc[17];            /* Param 6 de PLds : description    */
} LaminatorOption_t;

// ----------------------------------------------------
// Globals
// ----------------------------------------------------

MagInfo_t MagInfoCard;
LaminatorOption_t LamInfo;

char InkType[10];
char BlackIn[2];
char ColorSmooth[15];
char Soften[10];
char EjectCard[10];

static  int     page            = 0;              
static  int     cOrientation    = 0;
static  int     cSides          = 0;
static  int     tMargin         = 0;
static  int     lMargin         = 0;    
static  int     rMargin         = 0;
static  int     bMargin         = 0;

int Brightness[2], Contrast[2], SensibilityK[2], QualityK, SensibilityO[2];
int DetectAuto, StartDist;
int overlayRegion[2][8];

// ----------------------------------------------------
// Prototypes
// ----------------------------------------------------

// --- standard cups functions 

void            Setup                   ( void );
void            StartPage               ( const ppd_file_t * ppd, const cups_page_header_t * header, cups_raster_t * ras );
void            EndPage                 ( const cups_page_header_t * header );
void            Shutdown                ( void );
void            CancelJob               ( int sig );

// --- option management 

int             myGetIntOption          ( char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, int defaultValue );
char*           myGetStrOption          ( char *dest, char* optionName, int num_options, cups_option_t *options,    
                                                                                                ppd_file_t *ppd, const char* defaultStr );
char            myGetCharOption         ( char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, char defaultValue );
int             GetLaminationOption     ( LaminatorOption_t *lam_opt, int num_options, cups_option_t *options, ppd_file_t *ppd );
int             GetMagOption            ( MagInfo_t *mag, int num_options, cups_option_t *options, ppd_file_t *ppd );
int             GetPageSetupOption      ( int num_options, cups_option_t *options, ppd_file_t *ppd );
int             GetEvolisOption         ( int num_options, cups_option_t *options, ppd_file_t *ppd );
int             SendMagEncodingHeader   ( MagInfo_t* mag );
int             EvolisModelNumberFromPPD( char*  modelname );
int             ProcessSpecialCommands  ( const char* cmd, MagInfo_t* mag );
int             SetOverlayRegion        (  int region[8] );

// --- dsp
 
static  void    ShareRVBtoMem           ( long Height, long Width, long WidthLine, cups_raster_t * ras );
static  void    ShareRVBtoMemWindow     ( long Height, long Width, long WidthLine, cups_raster_t * ras, int isPortrait );
int             Sharpen                 ( unsigned char * lpMemIn, long Width, long Height );

// --- dbg

void            ViewInfo                ( const ppd_file_t * ppd, const cups_page_header_t * header );
void            ViewInfoPPD             ( const ppd_file_t * ppd );

// ----------------------------------------------------
//  void StartPage(const ppd_file_t * ppd,  
//                 const cups_page_header_t * header,
//                 cups_raster_t * ras)
//
// @author sa
// ----------------------------------------------------
void Setup(void)
{
    info("Setup()");
}

// ----------------------------------------------------
//  void StartPage(const ppd_file_t * ppd,  
//                 const cups_page_header_t * header,
//                 cups_raster_t * ras)
//
// @author sa
// ----------------------------------------------------
void StartPage(const ppd_file_t * ppd,  /* I - PPD file */
               const cups_page_header_t * header,   /* I - Page header */
               cups_raster_t * ras)
{
    int   putB = 0;
    int   overfb = 1;
    int   a;
    int   b;
    int   iColorSmooth; 
    int   face = RECTO;
    
#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
    
    struct sigaction action;    /* Actions for POSIX signals */
    
#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
    
#ifdef HAVE_SIGSET      /* Use System V signals over POSIX to avoid bugs */
    
    sigset(SIGTERM, CancelJob);
    
#elif defined(HAVE_SIGACTION)
    
    memset(&action, 0, sizeof(action));
    sigemptyset(&action.sa_mask);
    action.sa_handler = CancelJob;
    sigaction(SIGTERM, &action, NULL);
    
#else
    signal(SIGTERM, CancelJob);
#endif /* HAVE_SIGSET */
    
    info("StartPage()");
    
    if      (!(strcasecmp(ColorSmooth, "Classic")))     iColorSmooth = 1;
    else if (!(strcasecmp(ColorSmooth, "High")))        iColorSmooth = 2;
    else if (!(strcasecmp(ColorSmooth, "VeryHigh")))    iColorSmooth = 3;
    
	debug("Color %d", iColorSmooth);
	
    if ( (header->Duplex || cSides) && ((page & 1) == 0) )
    {       
        face = VERSO;
		//info("+++AT+FH:TRAITEMENT VERSO");
        
        if(page & 0x01) 
            overfb = 1; //front
        else
            overfb = 2; //back
    }
	else
	{
		//info("+++AT+FH:TRAITEMENT RECTO");
	}
    
    if  (   !(strcasecmp(header->MediaColor, "K"))      || !(strcasecmp(header->MediaColor, "KBL"))
         || !(strcasecmp(header->MediaColor, "KGO"))    || !(strcasecmp(header->MediaColor, "H"))
         || !(strcasecmp(header->MediaColor, "KGR"))    || !(strcasecmp(header->MediaColor, "KSI"))
         || !(strcasecmp(header->MediaColor, "KSC"))    || !(strcasecmp(header->MediaColor, "KA"))
         || !(strcasecmp(header->MediaColor, "KW"))     || !(strcasecmp(header->MediaColor, "KR")))
    {
        info("PRINT K");
        ConvertRVBtoK(&lpMem[4 * (dwSizeNeeded / 5)], dwSizeNeeded, 1016, 648, &lpMem[3 * (dwSizeNeeded / 5)], face);
        DBNC(0,1,0);
    }
    else if(!(strcasecmp(header->MediaColor, "KO")))
    {
        info("PRINT KO");
        ConvertRVBtoK(&lpMem[4 * (dwSizeNeeded / 5)], dwSizeNeeded, 1016, 648, &lpMem[3 * (dwSizeNeeded / 5)], face);
        DBNC(0,1,overfb);
    }
    else if(!(strcasecmp(header->MediaColor, "YMCKO")) || 
            !(strcasecmp(header->MediaColor, "YMCKOS")))
    {
        info("PRINT YMCKO");
        if(!(strcasecmp(header->MediaColor, "YMCKOS")))
        {
            if(DetectAuto)   // fhl - 01/08/08 - bug fixed 
            {
                GetFirstYMCDot(1016, 648 , &a, &b);
                output("\033Psp;%.4d\015", b);
                debug("DetectAuto \033Psp;%.4d\015", b);
            }
            else
            {
                output("\033Psp;%.4d\015", StartDist );
                debug("StartDist \033Psp;%.4d\015", StartDist );
            }
        }
        if (toupper(BlackIn[face]) != 'N')
            putB = 1;
        else
        {
            putB = 0;
            KinYMC(1016, 648);
        }
        DBNC(iColorSmooth,putB,overfb);
    }
    else if (!(strcasecmp(header->MediaColor, "ymckok")))
    {
        if (toupper(BlackIn[RECTO]) != 'N')
            putB = 1;
        else
        {
            putB = 0;
            KinYMC(1016, 648);
        } 
        
        if(header->Duplex || cSides)
        {
            if (page & 0x01)    
            {
                info("PRINT FRONT YMCKOK");
                DBNC(iColorSmooth,putB,overfb);
            }
            else
            {
                info("PRINT BACK YMCKOK");
                ConvertRVBtoK(&lpMem[4 * (dwSizeNeeded / 5)], dwSizeNeeded, header->cupsHeight, header->cupsWidth, &lpMem[3 * (dwSizeNeeded / 5)], VERSO);
                DBNC(0,1,0);
            }
        }
        else 
        {
            info("PRINT FRONT YMCKOK");
            DBNC(iColorSmooth,putB,overfb);
        }
    }
}

// ----------------------------------------------------
//  void EndPage(const cups_page_header_t * header) 
//
// @author sa
// ----------------------------------------------------
void EndPage(const cups_page_header_t * header) 
{
#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
    struct sigaction action;    /* Actions for POSIX signals */
#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
    
#ifdef HAVE_SIGSET      /* Use System V signals over POSIX to avoid bugs */
    sigset(SIGTERM, SIG_IGN);
#elif defined(HAVE_SIGACTION)
    memset(&action, 0, sizeof(action));
    sigemptyset(&action.sa_mask);
    action.sa_handler = SIG_IGN;
    sigaction(SIGTERM, &action, NULL);
#else
    signal(SIGTERM, SIG_IGN);
#endif /* HAVE_SIGSET */
    
    info("EndPage()");
}

// ----------------------------------------------------
//  void Shutdown(void)
//
// @author sa
// ----------------------------------------------------
void Shutdown(void)
{
    info("Shutdown()");
}

// ----------------------------------------------------
//  void CancelJob(int sig)     
//
// @author sa
// ----------------------------------------------------
void CancelJob(int sig)     
{
    info("CancelJob()");
    if(lpRecto)
    {
        debug("CancelJob lpRecto Free memory()");
        free(lpRecto);
        lpRecto = NULL;
    }
    if(lpVerso)
    {
        debug("CancelJob lpVerso Free memory()");
        free(lpVerso);
        lpVerso = NULL;
    }   
}

// ----------------------------------------------------
//  main()
//
// @authors sa, fhl
// ----------------------------------------------------

int main(int argc, char *argv[])        
{
    cups_raster_t           *ras;                   /* Raster stream for printing */
    cups_page_header_t      header;                 /* Page header from file */
    ppd_file_t              *ppd;                   /* PPD file */
    ppd_option_t            *option;                /* PPD option */
    cups_option_t           *options;               /* CUPS options */
    int                     model;                  /* Printer Model */
    int                     fd;                     /* File descriptor */
    int                     num_options;            /* Number of CUPS options */
    const char              *val;                   /* CUPS option value */
    char                    insert;                 /* insert mode */
    int                     hopper =0;              
    int                     feeder =0;
    unsigned int            index_copy, NbCopy;     /* Numbers of copies */
    char                    end_usb[65];
    int                     i;
    int                     appleorientation;       /* apple specific parameter */
    int                     appleduplex;            /* apple specific parameter */
    int                     isPortrait=0;           /* gestion du mode portrait */
    
    lpVerso = NULL;
    lpRecto = NULL;
    
    // Make sure status messages are not buffered...
    setbuf(stderr, NULL);
    
    info("Start Process");
    
    if (argc < 6 || argc > 7)
    {
        //We don't have the correct number of arguments; write an error message and return.
        printf("rastertoevolis - version %s\n", LVERSION);
        fatal("rastertoevolis job-id user title copies options [file]");
        return (STATUS);
    }
    
    //Open the page stream...
    if (argc == 7) 
    {
        if ((fd = open(argv[6], O_RDONLY)) == -1)
        {
            fatal("Unable to open raster file.");
            sleep(1);
            return (STATUS);
        }
    }
    else
    {
        fd = 0;
    }
    
    info( "[EVO] executing rastertoevolis version %s ...", LVERSION );
    ppd = ppdOpenFile(getenv("PPD"));
    
    model = EvolisModelNumberFromPPD( ppd->modelname );
    info("%s %d",ppd->modelname,model);
    
    ras = cupsRasterOpen(fd, CUPS_RASTER_READ);
    
    Setup();
    
    //---Get the STP options, if any
    num_options = cupsParseOptions(argv[5], 0, &options);
    
#ifdef MACOSX
    
    //------ MACOSX parameters
    
    appleduplex = myGetIntOption( "com.apple.print.PrintSettings.PMDuplexing..n.", num_options, options, ppd, -99 );
    debug("appleduplex : %d", appleduplex);
    
    // fhl - 08/07/08 - unfification linux/Mac
    if (appleduplex > 1) cSides = 1;
    else cSides = 0;
    
    appleorientation = myGetIntOption( "com.apple.print.PrintSettings.PMOrientation..n.", num_options, options, ppd, -99 );
    debug("appleorientation : %d", appleorientation);
    
	if (appleorientation>0) 
		isPortrait = ( appleorientation == 2 || appleorientation == 3 ); 
    else
		isPortrait = ( header.Orientation == 1 || header.Orientation == 3 || cOrientation == PORTRAIT );
#else
    isPortrait = ( header.Orientation == 1 || header.Orientation == 3 || cOrientation == PORTRAIT );
#endif // MacOS
    
    // --- Page Setup options

    if (! GetPageSetupOption( num_options, options, ppd ))
            info("Error: an error occured during the processing of the evolis options");

    // --- Ribbon settings
    
    if ((val = cupsGetOption("InkType", num_options, options)) != NULL)
        strcpy(InkType, val);
    else
        memset(InkType,0x00,9);
    
    debug("InkType : %s", InkType);
    
    // --- Evolis Printer Settings 

    if (! GetEvolisOption(num_options,options,ppd))
            info("Error: an error occured during the processing of the evolis options");
                
    // --- MAG OPTIONS
 
    if (! GetMagOption( &MagInfoCard, num_options, options, ppd ) )
            info("Error: an error occured during the parsing of the magnetic encoding options");
    
    // --- LAMINATION OPTIONS
        
    if (model == SECURION) 
        if (! GetLaminationOption( &LamInfo, num_options, options, ppd ) )
            info("Error: an error occured during the parsing of the lamination options");
        
    // --- PASSTHROUGH OPTION (PROCESSING)
    
    if (! ProcessPassthroughOption( num_options, options, ppd ) )
            info("Error: an error occured during the processing of the passthrough options");
        
    // ---- BEGIN OF PRINTING PROCESS
    
    if (cupsRasterReadHeader(ras, &header))
    {
        ViewInfo(ppd,&header);
        info("model : %d", model); // fhl - test - todo remove
        
        //Coherent info between manual add option and auto cups add option
        if(cOrientation)    header.Orientation  = cOrientation; 
        if(cSides != -1)    header.Duplex       = cSides; 
        
        /* ------------ */  
        
        //if(cupsRasterReadHeader(ras, &header))
        
        //  ViewInfo(ppd,&header);  // fhl - test - to comment
        
        if(model == QUANTUM) 
        {
            if      (!(strcasecmp(header.OutputType, "AutoH")))     hopper = 0;
            else if (!(strcasecmp(header.OutputType, "AlterH")))    hopper = 3;
            else if (!(strcasecmp(header.OutputType, "Tray1")))     hopper = 1;
            else if (!(strcasecmp(header.OutputType, "Tray2")))     hopper = 2;
            else                                                    hopper = 0;
            
            feeder = 5;
            if ( !(strcasecmp(header.MediaType, "Feeder1") ))  //header.MediaWeight == 1)
                feeder = 1;
            else if (!(strcasecmp(header.MediaType, "Feeder2"))) //(header.MediaWeight == 2)
                feeder = 2;
            else if (!(strcasecmp(header.MediaType, "Alter")))  //(header.MediaWeight == 3)
                feeder = 3;
            else if (!(strcasecmp(header.MediaType, "Auto")))  //(header.MediaWeight == 0)
                feeder = 0;
            
            if (feeder != 5)
            {
                info("Set Feeder and output hopper");
                output("\033Piem;%d;%d\015", feeder,hopper);
                //info("\033Piem;%d;%d\015", feeder,hopper);
            }
        }
        
        if (model == TATTOO) 
        {
            insert = '0';
            if (!(strcasecmp(header.MediaType, "Feeder"))) //header.MediaWeight == 1)
                insert = 'F';
            else if (!(strcasecmp(header.MediaType, "Manual"))) //header.MediaWeight == 2)
                insert = 'M';
            else if (!(strcasecmp(header.MediaType, "Auto"))) //header.MediaWeight == 3)
                insert = 'B';
            //else insert = '0';
            
            if (insert != '0')
            {
                info("Set Feeder");
                output("\033Pmi;%c\015", insert);
            }
        }
        
        // Process Card Test as needed...
        
        if ( !(strcasecmp(EjectCard, "NONE")) ) // || !(strcasecmp(EjectCard, "Keep"))) {
        {
            if (header.cupsColorSpace == CUPS_CSPACE_RGB || CUPS_CSPACE_CMYK )
                colorspace = CUPS_CSPACE_RGB;
            else
            {
                fatal("Fail CUPS_CSPACE_RGB ");
                goto quitmain;
            }
            
            //  dwSizeNeeded: Size of lpRecto in pixels
            if (header.cupsBitsPerPixel == 24)
            {
                //dwSizeNeeded = header.cupsHeight * header.cupsWidth * 5;
                dwSizeNeeded = 1016 * 648 *  5;
                if(lpRecto)
                {
                    debug("lpRecto Free memory");
                    free(lpRecto);
                    lpRecto = NULL;
                }
                lpRecto = malloc(dwSizeNeeded);
                if (lpRecto)
                    debug("lpRecto: size demands %d pixels... ", dwSizeNeeded);
                else
                {
                    fatal("Fail memory for lpRecto : %d pixels... ", dwSizeNeeded);
                    goto quitmain;
                } //End of lpRecto
                
                if(header.Duplex || cSides)
                {
                    //dwSizeNeeded = header.cupsHeight * header.cupsWidth * 5;
                    dwSizeNeeded = 1016 * 648 *  5;
                    if(lpVerso)
                    {
                        debug("lpVerso Free memory");
                        free(lpVerso);
                        lpVerso = NULL;
                    }
                    lpVerso = malloc(dwSizeNeeded);
                    if (lpVerso)
                        debug("lpVerso: size demands %d pixels... ", dwSizeNeeded);
                    else
                    {
                        fatal("Fail memory for lpVerso : %d pixels ", dwSizeNeeded);
                        goto quitmain;
                    }
                }// end of if Duplex
            }//end of cupRaster = 24
            else
            {
                info("header->cupsBitsPerPixel... : %d ", header.cupsBitsPerPixel);
                fatal("Fail Not mode 24 bits ");
                goto quitmain;
            }//End of lpRecto
            
            // Process pages as needed...
            
            // ====== BEGIN OF THE PRINTING PROCESS
            
            NbCopy = header.NumCopies;
            page = 0;
            do 
            {
                for ( NbCopy = header.NumCopies ; NbCopy > 0 ; NbCopy-- )
                {
                    page++;
                    if (model==SECURION) 
                    {
                        output("\033PLds;0;%c;%d;%d;%c;%s\015", LamInfo.lamType, LamInfo.lamSpeed, LamInfo.lamTemp, LamInfo.lamAlt, LamInfo.lamDesc);
                        output("\033PLr;0\015\033PLm;%c;%c\015", LamInfo.defLam, LamInfo.exitLam);
                    }
                    
                    //color panel
                                        
                    info("PAGE: %d %d", page, header.NumCopies);
                    
                    if ( (header.Duplex || cSides) && ((page & 1) == 0) ) // even page number and two-sided printing
                    {
                        info("SEQUENCE VERSO");
                        output("\033Sv\015");
                        if (model != TATTOO) 
                        {
                            info("Set color ribbon/panels temperature");
                            output("\033Pl;a;=;%d\015", Brightness[VERSO]);
                            output("\033Pc;a;=;%d\015", Contrast[VERSO]);
                            output("\033Pc;o;=;%d\015", SensibilityO[VERSO]);
                        }
                        info("Set black ribbon/panel temperature");
                        output("\033Pc;k;=;%d\015", SensibilityK[VERSO]);
                        // To improve output("\033Pr;%s\015",header.MediaColor);
                        SetOverlayRegion(overlayRegion[VERSO]);
                        lpMem = lpVerso;
                    }
                    else 
                    {
                        info("SEQUENCE RECTO");
                        output("\033Ss\015\033Sr\015");
                        if (model != TATTOO) 
                        {
                            info("Set color ribbon/panels temperature");
                            output("\033Pl;a;=;%d\015", Brightness[RECTO]);
                            output("\033Pc;a;=;%d\015", Contrast[RECTO]);
                            output("\033Pc;o;=;%d\015", SensibilityO[RECTO]);
                        }
                        info("Set black ribbon/panel temperature");
                        output("\033Pc;k;=;%d\015", SensibilityK[RECTO]);
                        output("\033Ps;k;=;%d\015", QualityK); // meme pour Recto/Verso
                        output("\033Pr;%s\015",header.MediaColor);
                        SetOverlayRegion(overlayRegion[RECTO]);
                        
                        for (i=0;i<3;i++)
                            if( MagInfoCard.Track[i].data[0] ) 
                            {
                                info("Download track %d", i+1);
                                output( "%s", MagInfoCard.Track[i].data );
                            }
                        lpMem = lpRecto;
                    }
                    
                    if ((header.cupsHeight * header.cupsWidth) == 658368 )
                        ShareRVBtoMem(header.cupsHeight, header.cupsWidth,  header.cupsBytesPerLine, ras);
                    else
                        ShareRVBtoMemWindow(header.cupsHeight, header.cupsWidth,  header.cupsBytesPerLine, ras, isPortrait);
                    
                    if (!(strcasecmp(Soften, "FALSE")))
                    {
                        Sharpen( &lpMem[0],648,1016);
                        Sharpen( &lpMem[1016*648],648,1016);
                        Sharpen( &lpMem[2*1016*648],648,1016);
                    }
                    
                    StartPage(ppd, &header, ras);
                    EndPage(&header);
                    
                    if (  (!(header.Duplex || cSides )) || ((page & 1) == 0) ) // even page number or not two-sided printing)
					{
						info("Eject card");
						output("\033Se\015");
					}
                }    // for NbCopy  
				info("--- next cupsRasterReadHeader please");
            }
            while (cupsRasterReadHeader(ras, &header));
			
			if ( (header.Duplex || cSides ) && (page & 1) ) // odd page number and two-sided printing
			{
				info("Eject card");
				output("\033Se\015");
			}
			
            memset(end_usb,0x00,64);
            if(fwrite( end_usb,sizeof(unsigned char),64,stdout) != 64) 
            {
                debug("Error: fails print end usb ...\n");
            }
            
        }//End test Card
        else
        {
            page = 1;
            do 
            {
                //cupsRasterReadPixels(ras, lpImgBuf, WidthLine)
            }
            while (cupsRasterReadHeader(ras, &header));
        
#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
            struct sigaction action;    
#endif 
#ifdef HAVE_SIGSET
            sigset(SIGTERM, CancelJob);
#elif defined(HAVE_SIGACTION)
            memset(&action, 0, sizeof(action));
            
            sigemptyset(&action.sa_mask);
            action.sa_handler = CancelJob;
            sigaction(SIGTERM, &action, NULL);
#else
            signal(SIGTERM, CancelJob);
#endif
            info("StartPage()");
            
            if (! ProcessSpecialCommands( EjectCard, &MagInfoCard ) )
                info("Error: an error occured during specific command processing");
            
            memset(end_usb,0x00,64);
            if(fwrite( end_usb,sizeof(unsigned char),64,stdout) != 64) 
                debug("Error: fails print end usb ...\n");
            EndPage(&header);
        }
        
    quitmain:;
        
        //Free memory...
        if(lpRecto)
        {
            debug("lpRecto Free memory");
            free(lpRecto);
            lpRecto = NULL;
        }
        if(lpVerso)
        {
            debug("lpVerso Free memory");
            free(lpVerso);
            lpVerso = NULL;
        }
        Shutdown();
        ppdClose(ppd);
        // Close the raster stream...
        cupsRasterClose(ras);
        if (fd != 0)
            close(fd);
        // If no pages were printed, send an error message...
        if (page == 0)
            fatal("ERROR: No pages found!\n");
        if (!STATUS) 
        {
            info("Job is done : successful operation(s)");
        } 
        else 
        {
            info("Error processing Job");
        }
        info("End of printing - Returning %d", STATUS);
        return (STATUS);
    }
}

// ----------------------------------------------------
//  static void ShareRVBtoMemWindow(long Height, long Width, long WidthLine, cups_raster_t * ras, int isPortrait )
//
// @author sa
// ----------------------------------------------------
// pMem ppV
//                       ___________
//                      |           |
//                      |           |
//                      |   Yellow  |
//                      |           |
//                      |___________|
//                      |           |
//                      |           |
//                      |   Magenta |
//                      |           |
//                      |___________|
//                      |           |
//                      |           |
//                      |   Cyan    |
//                      |           |
//                      |___________|
//                      |           |
//                      |           |
//                      |   Black   |
//                      |           |
//                      |___________|
//                      |           |
//                      |           |
//                      |   Grey    |
//                      |           |
//                      |___________|
// ----------------------------------------------------
static void ShareRVBtoMem(long Height, long Width, long WidthLine, cups_raster_t * ras)
{
    //Pointeur sur image buffer
    unsigned char *lpImgBuf, *lpYellow, *lpMagenta, *lpCyan, *lpBlack, *lpGray;
    //valeur RVB courante & pr
    unsigned char red, green, blue, r, g, b;
    //index pour scruter le fichier bitmap
    long index_line, index_dot;
    long  stpt = 0;
    long  offx = 0, offy = 0;
    
    lpImgBuf = malloc(WidthLine);
    if (!lpImgBuf) 
    {
        fatal("ShareRVBtoMem fail to book memory: size requested %d bytes... \n", WidthLine);
        return;
    }
    lpYellow = &lpMem[0];
    //memset(lpYellow,0x00,Width * Height);
    memset(lpYellow,0x00,1016*648);
    lpMagenta = &lpMem[Width * Height];
    //memset(lpMagenta,0x00,Width * Height);
    memset(lpMagenta,0x00,1016*648);
    lpCyan = &lpMem[2 * Width * Height];
    //memset(lpCyan,0x00,Width * Height);
    memset(lpCyan,0x00,1016*648);  
    lpBlack = &lpMem[3 * Width * Height];
    //memset(lpBlack,0x00,Width * Height);
    memset(lpBlack,0x00,1016*648);
    lpGray = &lpMem[4 * Width * Height];
    //memset(lpGray,0xFF,Width * Height);
    memset(lpGray,0xFF,1016*648);
    
    ////////////////
    //CAS PORTRAIT//
    ////////////////
    if (Height >= Width) 
    {
        info("Portrait");
        //Pour toutes les lignes de l'image
        for (index_line = 0; index_line < Height; index_line++) 
        {
            if (cupsRasterReadPixels(ras, lpImgBuf, WidthLine) == WidthLine) 
            {
                r = 0xFF;
                g = 0xFF;
                b = 0xFF;
                stpt = 0;
                //Pour tous les points de la ligne
                for (index_dot = Width; index_dot > 0; index_dot--) 
                {
                    red = lpImgBuf[(index_dot * 3)];
                    green = lpImgBuf[(index_dot * 3) + 1];
                    blue = lpImgBuf[(index_dot * 3) + 2];
                    lpGray[(index_line * 648) + index_dot] = (unsigned char) RgbToGray(red, green, blue);
                    if (blue == 0x00 && green == 0x00 && red == 0x00) 
                    {
                        if (stpt == 0)  
                        {
                            //Erreur sur le premier point blanc par defaut
                            lpYellow[(index_line * 648) + index_dot] = 0x00;
                            lpMagenta[(index_line * 648) + index_dot] = 0x00;
                            lpCyan[(index_line * 648) + index_dot] = 0x00;
                            lpBlack[(index_line * 648) + index_dot] = 0xFF;
                            //premier point trait...
                            stpt = 1;
                        }
                        else 
                        {
                            //Cas point noir on met la couleur du point precedent
                            lpYellow[(index_line * 648) + index_dot] = ((0xFF - b));
                            lpMagenta[(index_line * 648) + index_dot] = ((0xFF - g));
                            lpCyan[(index_line * 648) + index_dot] = ((0xFF - r));
                            lpBlack[(index_line * 648) + index_dot] = 0xFF;
                        }
                    }
                    else 
                    {
                        //Sauvegarde de la valeur precedente
                        r = red;
                        g = green;
                        b = blue;
                        //Traitement particulier pour le premier point
                        lpYellow[(index_line * 648) + index_dot] = ((0xFF - b));
                        lpMagenta[(index_line * 648) + index_dot] = ((0xFF - g));
                        lpCyan[(index_line * 648) + index_dot] = ((0xFF - r));
                        lpBlack[(index_line * 648) + index_dot] = 0x00;
                        //premier point traite
                        stpt = 1;
                    }
                }   // for point de la line
            }   // test read line = cupsRasterReadPixels
        }   //for ligne
    }   //end if H>W
    /////////////////
    //CAS LANDSCAPE//
    /////////////////
    else 
    {
        info("Landscape");
        for (index_line = (Height); index_line > 0; index_line--)
        {
            r = 0xFF;
            g = 0xFF;
            b = 0xFF;
            stpt = 0;
            if (cupsRasterReadPixels(ras, lpImgBuf, WidthLine) == WidthLine) 
            {
                for (index_dot = 0; index_dot < Width; index_dot++) 
                {
                    red = lpImgBuf[(index_dot * 3)];
                    green = lpImgBuf[(index_dot * 3) + 1];
                    blue = lpImgBuf[(index_dot * 3) + 2];
                    lpGray[(index_line) + (index_dot * Height)] = (unsigned char) RgbToGray(red, green, blue);
                    if(blue == 0x00 && green == 0x00 && red == 0x00) 
                    {
                        if (stpt == 0)
                        {
                            //Erreur sur le premier point blanc par defaut
                            lpYellow[(index_line) + (index_dot * Height)] = 0x00;
                            lpMagenta[(index_line) + (index_dot * Height)] = 0x00;
                            lpCyan[(index_line) + (index_dot * Height)] = 0x00;
                            lpBlack[(index_line) + (index_dot * Height)] = 0xFF;
                            //premier point traite
                            stpt = 1;
                        }
                        else 
                        {
                            //Cas point noir on met la couleur du point precedent
                            lpYellow[(index_line) + (index_dot * Height)] = ((0xFF - b));
                            lpMagenta[(index_line) + (index_dot * Height)] = ((0xFF - g));
                            lpCyan[(index_line) + (index_dot * Height)] = ((0xFF - r));
                            lpBlack[(index_line) + (index_dot * Height)] = (0xFF);
                        }
                    }
                    else 
                    {
                        //Sauvegarde de la valeur precedente
                        r = red;
                        g = green;
                        b = blue;
                        //Traitement particulier pour le premier point
                        lpYellow[(index_line) + (index_dot * Height)] = ((0xFF - b));
                        lpMagenta[(index_line) + (index_dot * Height)] = ((0xFF - g));
                        lpCyan[(index_line) + (index_dot * Height)] = ((0xFF - r));
                        lpBlack[(index_line) + (index_dot * Height)] = 0x00;
                        //premier point traite
                        stpt = 1;
                    }
                }   // for point de la line
            }   // test read line
        }       //for ligne
    }           //end else if H>W
    free(lpImgBuf);
} 

// ----------------------------------------------------
//  static void ShareRVBtoMemWindow(long Height, long Width, long WidthLine, cups_raster_t * ras, int isPortrait )
//
// @author sa
// ----------------------------------------------------

static void ShareRVBtoMemWindow(long Height, long Width, long WidthLine, cups_raster_t * ras, int isPortrait )
{
    unsigned char *lpImgBuf, *lpYellow, *lpMagenta, *lpCyan, *lpBlack, *lpGray;         // Pointeur sur image buffer
    unsigned char red, green, blue, r, g, b;                                            // valeur RVB courante & presente
    long index_line, index_dot;                                                         // index pour scruter le fichier bitmap
    long  stpt = 0;                                                                     // Booleen indiquant le premier bit de la chaine
    long  offx = 0, offy = 0;
    long  Hfix = 1016, Wfix = 648;
    long  dot = 0, line =0;
    
    lpImgBuf = malloc(WidthLine);
    if (!lpImgBuf) {
        fatal("ShareRVBtoMem fail to book memory: size requested %d bytes... \n", WidthLine);
        return;
    }
    
    lpYellow = &lpMem[0];
    memset(lpYellow,0x00,1016*648);
    lpMagenta = &lpMem[1016*648];
    memset(lpMagenta,0x00,1016*648);
    lpCyan = &lpMem[2 * 1016*648];
    memset(lpCyan,0x00,1016*648);
    lpBlack = &lpMem[3 * 1016*648];
    memset(lpBlack,0x00,1016*648);
    lpGray = &lpMem[4 * 1016*648];
    memset(lpGray,0xFF,1016*648);
    
    if ( isPortrait ) 
    {
        if(Width%2) 
        {
            cupsRasterReadPixels(ras, lpImgBuf, WidthLine);
            //info("Read odd line ");
            Width--;
        }
        if(Width > 648)
            offx = (Width - 648)/2;
        else
            offx = 0;
        
        if(Height > 1016)
            offy = (Height - 1016)/2;
        else
            offy = 0;
        
        Hfix = 1016;
        Wfix = 648;
    }
    else 
    {
        if (Height%2) 
        {
            cupsRasterReadPixels(ras, lpImgBuf, WidthLine);
            //info("Read odd line ");
            Height--;
        }
        if(Width > 1016)
            offx = (Width - 1016)/2;
        else
            offx = 0;
        
        if(Height > 648)
            offy = (Height - 648)/2;
        else
            offy = 0; 
        Hfix = 648;
        Wfix = 1016;
    }
    
    //info(" offx : %d offy : %d", offx, offy);
    ////////////////
    //CAS PORTRAIT//
    ////////////////
    //if (Height >= Width)
    if ( isPortrait )
    {
        info("Portrait");
        for (index_line = (Height); index_line > (Height - offy); index_line--) 
        {
            cupsRasterReadPixels(ras, lpImgBuf, WidthLine);
        }
        //info(" H : %d  W: %d",Hfix,Wfix);
        //Pour toutes les lignes de l'image
        line = 0;
        for (index_line; index_line > offy; index_line--) 
        {
            if (cupsRasterReadPixels(ras, lpImgBuf, WidthLine) == WidthLine) 
            {
                r = 0xFF;
                g = 0xFF;
                b = 0xFF;
                stpt = 0;
                //Pour tous les points de la ligne
                dot = 648;
                for (index_dot = (Width-offx); index_dot > offx; index_dot--) 
                {
                    red = lpImgBuf[(index_dot * 3)];
                    green = lpImgBuf[(index_dot * 3) + 1];
                    blue = lpImgBuf[(index_dot * 3) + 2];
                    lpGray[(line*648) + (dot) ] = (unsigned char) RgbToGray(red, green, blue);
                    if (blue == 0x00 && green == 0x00 && red == 0x00) 
                    {
                        if (stpt == 0)  
                        {
                            //Erreur sur le premier point blanc par defaut
                            lpYellow[(line*648) + (dot)] = 0x00;
                            lpMagenta[(line*648) + (dot)] = 0x00;
                            lpCyan[(line*648) + (dot)] = 0x00;
                            lpBlack[(line*648) + (dot)] = 0xFF;
                            //premier point traite
                            stpt = 1;
                        }
                        else 
                        {
                            //Cas point noir on met la couleur du point precedent
                            lpYellow[(line*648) + (dot)] = ((0xFF - b));
                            lpMagenta[(line*648) + (dot)] = ((0xFF - g));
                            lpCyan[(line*648) + (dot)] = ((0xFF - r));
                            lpBlack[(line*648) + (dot)] = 0xFF;
                        }
                    }
                    else 
                    {
                        //Sauvegarde de la valeur precedente
                        r = red;
                        g = green;
                        b = blue;
                        //Traitement particulier pour le premier point
                        lpYellow[(line*648) + (dot)] = ((0xFF - b));
                        lpMagenta[(line*648) + (dot)] = ((0xFF - g));
                        lpCyan[(line*648) + (dot)] = ((0xFF - r));
                        lpBlack[(line*648) + (dot)] = 0x00;
                        //premier point traite
                        stpt = 1;
                    }
                    dot--;
                }   // for point de la line
            }   // test read line = cupsRasterReadPixels
            line++;
        }       //for ligne
        for (index_line ; index_line > 0; index_line--) 
        {
            if(cupsRasterReadPixels(ras, lpImgBuf, WidthLine) != WidthLine) 
            {
                debug("Erreur de lecture line CUPS15");
            }
        }
    }       //end if H>W
    /////////////////
    //CAS LANDSCAPE//
    /////////////////
    else 
    {
        info("Landscape");
        //info("index_line: %d",Height);
        for (index_line = (Height); index_line > (Height - offy); index_line--) 
        {
            if(cupsRasterReadPixels(ras, lpImgBuf, WidthLine) != WidthLine) 
            {
                debug("erreur ");
            }
        }
        line = 648;//Hfix;
        for (index_line ; index_line > offy; index_line--) 
        {
            r = 0xFF;
            g = 0xFF;
            b = 0xFF;
            stpt = 0;
            if (cupsRasterReadPixels(ras, lpImgBuf, WidthLine) == WidthLine) 
            {
                dot = 0;
                for (index_dot = offx; index_dot < (Width - offx); index_dot++) 
                {
                    red = lpImgBuf[(index_dot * 3)];
                    green = lpImgBuf[(index_dot * 3) + 1];
                    blue = lpImgBuf[(index_dot * 3) + 2];
                    lpGray[(line) + (dot * 648)] = (unsigned char) RgbToGray(red, green, blue);
                    if(blue == 0x00 && green == 0x00 && red == 0x00) 
                    {
                        if (stpt == 0) 
                        {
                            //Erreur sur le premier point blanc par defaut
                            lpYellow[(line) + (dot * 648)] = 0x00;
                            lpMagenta[(line) + (dot * 648)] = 0x00;
                            lpCyan[(line) + (dot * 648)] = 0x00;
                            lpBlack[(line) + (dot * 648)] = 0xFF;
                            //premier point traite
                            stpt = 1;
                        }
                        else 
                        {
                            //Cas point noir on met la couleur du point precedent
                            lpYellow[(line) + (dot * 648)] = ((0xFF - b));
                            lpMagenta[(line) + (dot * 648)] = ((0xFF - g));
                            lpCyan[(line) + (dot * 648)] = ((0xFF - r));
                            lpBlack[(line) + (dot * 648)] = (0xFF);
                        }
                    }
                    else 
                    {
                        //Sauvegarde de la valeur precedente
                        r = red;
                        g = green;
                        b = blue;
                        //Traitement particulier pour le premier point
                        lpYellow[(line) + (dot * 648)] = ((0xFF - b));
                        lpMagenta[(line) + (dot * 648)] = ((0xFF - g));
                        lpCyan[(line) + (dot * 648)] = ((0xFF - r));
                        lpBlack[(line) + (dot * 648)] = 0x00;
                        //premier point traite
                        stpt = 1;
                    }
                    dot++;
                }   // for point de la line
            }   // test read line
            line--;
        }       //for ligne
        for (index_line; index_line > 0; index_line--) 
        {
            if(cupsRasterReadPixels(ras, lpImgBuf, WidthLine) != WidthLine) 
            {
                debug("erreur ");
            }
        }
    }           //end else if H>W
    free(lpImgBuf);
}



//Sharpen (medium)
int SharpenPattern2[5][5] = {
    {-1,-1,-1,-1,-1},
    {-1,-1,-1,-1,-1},
    {-1,-1,49,-1,-1},
    {-1,-1,-1,-1,-1},
    {-1,-1,-1,-1,-1}
};

// ----------------------------------------------------
//  int Sharpen(unsigned char * lpMemIn, long Width, long Height)
//
// @author sa
// ----------------------------------------------------

int Sharpen(unsigned char * lpMemIn, long Width, long Height)
{
    long    index_line = 0,index_dot = 0;
    unsigned char *lpbDataOut , *lpbData;
    
    int  final = 0;
    int x= 0,y=0;
    int divisionFactor = 0;
    
    lpbDataOut = NULL;
    lpbDataOut = malloc(Width * Height);
    if (lpbDataOut)
        debug("lpDataOut: size demands %d pixels... ", Width * Height);
    else
        fatal("Fail memory for lpDataOut : %d pixels ", Width * Height);
    
    for(y=-2; y<=2; y++)     // and for each pixel around our
        for(x=-2; x<=2; x++)   //  "hot pixel"...
            divisionFactor += SharpenPattern2[x+2][y+2];
    
    //lpbData ppV mem bmp brute
    lpbData = &lpMemIn[0];
    //lpbDataOut = &lpMemOut[0];
    for(index_line = 2; index_line < Height -2; index_line++)
        for(index_dot = 2 ; index_dot < Width -2; index_dot++)
        {
            //gridCounter=0;      // reset some values
            final = 0;
            for(y=-2; y<=2; y++)     // and for each pixel around our
                for(x=-2; x<=2; x++)   //  "hot pixel"...
                    final += lpbData[(index_dot + x) + ((index_line + y) * Width)] * SharpenPattern2[y+2][x+2];
            
            // and put it back into the right range
            final /= divisionFactor;
            //iMemFilter[index_dot + (index_line * Width)] = final;
            lpbDataOut[(index_dot) + ((index_line) * Width)] = final;
            if(final > 255)
                lpbDataOut[(index_dot) + ((index_line) * Width)] = 0xFF;
            else if(final < 0)
                lpbDataOut[(index_dot) + ((index_line) * Width)] = 0x00;
        }
    index_dot = Height * Width;
    for(index_line = 0; index_line < index_dot; index_line++)
        lpbData[index_line] =  lpbDataOut[index_line];
    
    //Free memory...
    if(lpbDataOut)
    {
        debug("lpbDataOut Free memory");
        free(lpbDataOut);
        lpbDataOut = NULL;
    } 
    
    
    return(1);
}

// ----------------------------------------------------
//  void ViewInfo(const ppd_file_t * ppd, const cups_page_header_t * header)
//
// @brief debug
// @author sa
// @param dest :  ppd param pointer and cups header info
// ----------------------------------------------------
void ViewInfo(const ppd_file_t * ppd, const cups_page_header_t * header)
{
    
    info("-----------------------------------");
    info("header->cupsWidth... : %d ", header->cupsWidth);
    
    info("header->cupsHeight... : %d ", header->cupsHeight);
    
    info("header->MediaClass... : %s ", header->MediaClass);
    info("header->MediaColor... : %s ", header->MediaColor);
    info("header->MediaType... : %s ", header->MediaType);  
    info("header->OutputType... : %s ", header->OutputType);
    info("header->CutMedia... : %d ", header->CutMedia);
    
    info("header->AdvanceDistance... : %s ", header->AdvanceDistance);
    info("header->AdvanceMedia... : %d ", header->AdvanceMedia);    // Width of page image in pixels //
    info("header->Collate... : %d ", header->Collate);  // Width of page image in pixels //
    
    info("header->Duplex... : %d ", header->Duplex);    
    info("header->HWResolution... : %d %d ", header->HWResolution[0],header->HWResolution[1]);
    info("header->ImagingBoundingBox... : %d %d %d %d", header->ImagingBoundingBox[0],header->ImagingBoundingBox[1],header->ImagingBoundingBox[2],header->ImagingBoundingBox[3]);   
    info("header->InsertSheet... : %d ", header->InsertSheet);  // Width of page image in pixels //
    info("header->Jog... : %d ", header->Jog);  // Width of page image in pixels //
    info("header->LeadingEdge... : %d ", header->LeadingEdge);  // Width of page image in pixels //
    info("header->Margins... : %d , %d", header->Margins[0],header->Margins[1]);    
    info("header->ManualFeed... : %d ", header->ManualFeed);
    info("header->MediaPosition... : %d ", header->MediaPosition);  
    info("header->MediaWeight... : %d ", header->MediaWeight);
    info("header->MirrorPrint... : %d ", header->MirrorPrint);  // Width of page image in pixels //
    info("header->NegativePrint... : %d ", header->NegativePrint);  // Width of page image in pixels //
    info("header->NumCopies... : %d ", header->NumCopies);  
    info("header->Orientation... : %d ", header->Orientation);
    info("header->OutputFaceUp... : %d ", header->OutputFaceUp);    // Width of page image in pixels //
    info("header->PageSize... : %d ,%d ", header->PageSize[0], header->PageSize[1]);
    info("header->Separations... : %d ", header->Separations);  // Width of page image in pixels //
    info("header->TraySwitch... : %d ", header->TraySwitch);
    info("header->Tumble... : %d ", header->Tumble);    
    info("header->cupsWidth... : %d ", header->cupsWidth);
    info("header->cupsHeight... : %d ", header->cupsHeight);
    info("header->MediaType... : %d ", header->cupsMediaType);
    info("header->cupsBitsPercolor... : %d ", header->cupsBitsPerColor);    
    info("header->cupsBitsPerPixel... : %d ", header->cupsBitsPerPixel);    
    info("header->cupsBitsPerline... : %d ", header->cupsBytesPerLine); 
    info("header->cupsColorOrder... : %d ", header->cupsColorOrder);    
    info("header->cupsColorSpace... : %d ", header->cupsColorSpace);    
    info("header->cupsCompression... : %d ", header->cupsCompression);  
    info("header->cupsRowCount... : %d ", header->cupsRowCount);    
    info("header->cupsRowFeed... : %d ", header->cupsRowFeed);  
    info("header->cupsRowStep... : %d ", header->cupsRowStep);  
    info("header->cupsColorSpace... : %d ", header->cupsColorSpace);    
    info("header->cupsColorOrder... : %d ", header->cupsColorOrder);    
    info("header->InsertSheet... : %d ", header->InsertSheet);  // Width of page image in pixels //
    info("header->Jog... : %d ", header->Jog);  // Width of page image in pixels //
    info("header->LeadingEdge... : %d ", header->LeadingEdge);  // Width of page image in pixels //
    info("header->ManualFeed... : %d ", header->ManualFeed);    // Width of page image in pixels //
    info("header->MediaPosition... : %d ", header->MediaPosition);  // Width of page image in pixels //
    info("header->MediaWeight... : %d ", header->MediaWeight);  // Width of page image in pixels //
    info("header->MirrorPrint... : %d ", header->MirrorPrint);  // Width of page image in pixels //
    info("header->NegativePrint... : %d ", header->NegativePrint);  // Width of page image in pixels //
    info("header->OutputFaceUp... : %d ", header->OutputFaceUp);    // Width of page image in pixels //
    info("header->Separations... : %d ", header->Separations);  // Width of page image in pixels //
    info("-----------------------------------");
}

// ----------------------------------------------------
//  void ViewInfoPPD(const ppd_file_t * ppd)
//
// @brief debug
// @author sa
// @param dest :  ppd param pointer
// ----------------------------------------------------
void ViewInfoPPD(const ppd_file_t * ppd)
{
    info("PPD->Langage Level # - %d", ppd->language_level);
    
    
    info("PPD->color_device # - %d", ppd->color_device);
    info("PPD->variable_sizes # - %d", ppd->variable_sizes);
    info("PPD->accurate_screens # - %d", ppd->accurate_screens);
    info("PPD->contone_only # - %d", ppd->contone_only);
    info("PPD->landscape # - %d", ppd->landscape);
    info("PPD->model_number # - %d", ppd->model_number);
    info("PPD->manual_copies # - %d", ppd->manual_copies);
    info("PPD->throughput # - %d", ppd->throughput);
    info("PPD->colorspace # - %d", ppd->colorspace);
    info("PPD->num_emulations # - %d", ppd->num_emulations);
    info("PPD->emulation # - %s", ppd->emulations);
    
    info("PPD->jcl_begin # - %s", ppd->jcl_begin);
    info("PPD->pjcl_end # - %s", ppd->jcl_end);
    info("PPD->lang_encoding # - %s", ppd->lang_encoding);
    
    info("PPD->lang_version # - %s", ppd->lang_version);
    info("PPD->modelname # - %s", ppd->modelname);
    info("PPD->ttrasterizer # - %s", ppd->ttrasterizer);
    info("PPD->manufacturer # - %s", ppd->manufacturer);
    info("PPD->product # - %s", ppd->product);
    info("PPD->nickname # - %s", ppd->nickname);
    info("PPD->shortnickname # - %s", ppd->shortnickname);
    info("PPD->num_groups # - %d", ppd->num_groups);
    info("PPD->num_sizes # - %d", ppd->num_sizes);
    info("PPD->sizes[0] # - %d, %s, %f, %f, %f, %f, %f, %f", ppd->sizes[0].marked,ppd->sizes[0].name,
         ppd->sizes[0].width,ppd->sizes[0].length,ppd->sizes[0].left,ppd->sizes[0].right,ppd->sizes[0].top );
    
    
    info("PPD->custom_min # - %f, %f", ppd->custom_min[0],ppd->custom_min[1]);
    info("PPD->custom_max # - %f, %f", ppd->custom_max[0],ppd->custom_max[1]);
    info("PPD->custom_margins # - %f,%f,%f,%f", ppd->custom_margins[0],ppd->custom_margins[1]);
    info("PPD->num_consts # - %d", ppd->num_consts);
    
    info("PPD->num_fonts # - %d", ppd->num_fonts);
    info("PPD->num_profiles # - %d", ppd->num_profiles);
    info("PPD->num_filters # - %d", ppd->num_filters);
    info("PPD->filters # - %s", ppd->filters[0]);
    
    info("PPD->flip_duplex # - %d", ppd->flip_duplex);
}

// ----------------------------------------------------
//  int myGetIntOption( char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, int defaultValue )
//
// @brief get an option (int) from cups then ppd option then default
// @author fhl
// @param dest :  the name of the option, the ppd/cups params, the default value
// @return the resulting int
// ----------------------------------------------------
int myGetIntOption( char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, int defaultValue )
{
    const char      *val;                   /* CUPS option value */ 
    ppd_option_t    *option;                /* PPD option */
    
    if ( (val = cupsGetOption( optionName, num_options, options)) != NULL)
        return  atoi(val);
    else if ((option = ppdFindOption(ppd, optionName)) != NULL)
        return atoi(option->defchoice);
    else
        return defaultValue;
}

// ----------------------------------------------------
//  char myGetCharOption( char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, char defaultValue )
//
// @brief get an option (char) - the first char of cups then ppd option then default
// @author fhl
// @param dest :  the name of the option, the ppd/cups params, the default value
// @return the resulting char
// ----------------------------------------------------
char myGetCharOption( char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, char defaultValue )
{
    const char      *val;                   /* CUPS option value */ 
    ppd_option_t    *option;                /* PPD option */
    
    if ( (val = cupsGetOption( optionName, num_options, options)) != NULL)
        return val[0];
    else if ((option = ppdFindOption(ppd, optionName)) != NULL)
        return (option->defchoice)[0];
    else
        return defaultValue;
}

// ----------------------------------------------------
//  char* myGetStrOption( char *dest, char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, const char* defaultStr )
//
// @brief get an option (string) from cups then ppd option then default
// @author fhl
// @param dest : the resulting string, the name of the option, the ppd/cups params, the default value
// @return the resulting string pointer
// ----------------------------------------------------
char* myGetStrOption( char *dest, char* optionName, int num_options, cups_option_t *options, ppd_file_t *ppd, const char* defaultStr )
{
    const char      *val;                   /* CUPS option value */ 
    ppd_option_t    *option;                /* PPD option */
    
    if ( (val = cupsGetOption( optionName, num_options, options)) != NULL)
        return strcpy(dest, val);
    else if ((option = ppdFindOption(ppd, optionName)) != NULL)
        return strcpy(dest, option->defchoice);
    else
        return strcpy(dest, defaultStr);
}

// ----------------------------------------------------
//  int EvolisModelNumberFromPPD( char*  modelname )
//
// @brief see return ... 
// @author fhl
// @param take ppd header model description
// @return number associated to our printer model (see evolis.h)            
// ----------------------------------------------------
int EvolisModelNumberFromPPD( char*  modelname )
{
    if      (!strcasecmp(modelname,"EVOLIS Pebble Card Printer"))       return PEBBLE;
    else if (!strcasecmp(modelname,"EVOLIS Tattoo Card Printer"))       return TATTOO;
    else if (!strcasecmp(modelname,"EVOLIS Quantum Card Printer"))      return QUANTUM;
    else if (!strcasecmp(modelname,"EVOLIS Dualys Card Printer"))       return DUALYS;
    else if (!strcasecmp(modelname,"EVOLIS Securion Card Printer"))     return SECURION;    // fhl - 11/06/08
    else if (!strcasecmp(modelname,"EVOLIS Tattoo2 Card Printer"))      return TATTOO2;     // fhl - 11/06/08 - idem que PEBBLE
    else                                                                return PEBBLE;      // fhl - 09/07/08 - default model
}

// ----------------------------------------------------
//  int     SendMagEncodingHeader   ( MagInfo_t* mag );
//
// @brief prepare printer for encoding by sending mag parameters 
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ----------------------------------------------------
int     SendMagEncodingHeader   ( MagInfo_t* mag )
{       
    int i;
    
    if (!(strcasecmp(mag->Coercivity, "HICO")))
        output("\033Pmc;h\015");    
    else
        output("\033Pmc;l\015");
    
    for ( i=0 ; i<3 ; i++ )
    {
        output("\033Pmt;%d;%d\015",i+1,mag->Track[i].type);
        
        if ( mag->Track[i].density == 0 ) // default 
        {
            if ( mag->Track[i].type == MAGTYPE_C2_75  ) output( "\033Pmd;%d;%d\015", i+1, 75 );
            if ( mag->Track[i].type == MAGTYPE_C2_210 ) output( "\033Pmd;%d;%d\015", i+1, 210 );
        }
        else
            output( "\033Pmd;%d;%d\015", i+1, mag->Track[i].density );
        
        debug( "Mag Density Track%d : %d",  i+1, mag->Track[i].density );
    }       
    return 1; // no parameters checking ... yet
}

// ----------------------------------------------------
// int      ProcessSpecialCommands  ( const char* cmd, MagInfo_t* mag )
//
// @brief prepare printer for encoding by sending mag parameters 
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ----------------------------------------------------
int     ProcessSpecialCommands  ( const char* cmd, MagInfo_t* mag )
{
    int i;
    if (!(strcasecmp(cmd, "ISMART")))
    {
        info("Inserting smart card.");
        output("\033Sis\015");
    }
    else if (!(strcasecmp(cmd, "ICTLESS")))
    {
        info("Inserting contactless card.");
        output("\033Sic\015");
    }
    else if (!(strcasecmp(cmd, "EJECT")))
    {
        info("Ejecting card.");
        output("\033Se\015");
    }
    else if (!(strcasecmp(cmd, "REJECT")))
    {
        info("Rejecting card.");
        output("\033Ser\015");
    }
    else if (!(strcasecmp(cmd, "TEST")))
    {
        info("Self test card.");
        output("\033St\015");
    }
    else if (!(strcasecmp(cmd, "CLEAN")))
    {
        info("Self cleaning card.");
        output("\033Scp\015");
    }
    else if (!(strcasecmp(cmd, "AJUST")))
    {
        info("Self adjusment.");
        output("\033Sa\015");
    }
    else if (!(strcasecmp(cmd, "SELFMAG")))
    {
        info("Self magnetic test card.");
        output("\033Stt;m\015");
    }
    else if (!(strcasecmp(cmd, "TestTech")))   // fhl - 26/06/08 - ajout
    {
        info("Technical Test Card");
        output("\033Stt\015");
    }
    else if (!(strcasecmp(cmd, "ONLYENCO")))
    {
        info("Only encoding.");
        
        output("\033Ss\015");
        for (i=0;i<3;i++)
            if( MagInfoCard.Track[i].data[0] ) 
            {
                info("Download track %d", i+1);
                output( "%s", MagInfoCard.Track[i].data );
            }
            
        output("\033Smw\015");
        if (!mag->KeepCard) 
        {
            info("Eject card after encoding.");
            output("\033Se\015");
        }
        else 
        {
            info("Keep card after encoding.");
            output("\033Ste\015");
        }
    }
    return 1;  // todo 
}

// ----------------------------------------------------
// int GetLaminationOption( LaminatorOption_t *lam_opt, int num_options, cups_option_t *options, ppd_file_t *ppd )
//
// @brief parse lamination options
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ----------------------------------------------------
int GetLaminationOption( LaminatorOption_t *lam_opt, int num_options, cups_option_t *options, ppd_file_t *ppd )
{
    // fhl - 11/06/08 - Parsing Lamination Option : securion        
    
    lam_opt->defLam = myGetCharOption( "DefLam", num_options, options, ppd, '0' );
    
    if ( (lam_opt->defLam!='0') && (lam_opt->defLam!='r') && 
         (lam_opt->defLam!='v') && (lam_opt->defLam!='b') ) 
         lam_opt->defLam='0';           
    info("DefLam : %c", lam_opt->defLam);
    
    lam_opt->exitLam = myGetCharOption( "ExitLam", num_options, options, ppd, '0' );
    
    if ( (lam_opt->exitLam!='0') && (lam_opt->exitLam!='r') && 
         (lam_opt->exitLam!='v') && (lam_opt->exitLam!='b') ) 
         lam_opt->exitLam='0'; 
    info("ExitLam : %c", lam_opt->exitLam);
    
    lam_opt->lamType = myGetCharOption( "LamType", num_options, options, ppd, '0' );
    
    if ( (lam_opt->lamType!='0') && (lam_opt->lamType!='1') && (lam_opt->lamType!='2') ) 
        lam_opt->lamType='0'; 
    info("LamType : %c", lam_opt->lamType);
    
    lam_opt->lamAlt = myGetCharOption( "LamAlt", num_options, options, ppd, '0' );
    
    if ( (lam_opt->lamAlt!='0') && (lam_opt->lamAlt!='1') && (lam_opt->lamAlt!='2') ) 
        lam_opt->lamAlt='0'; 
    info("LamAlt : %c", lam_opt->lamAlt);
    
    lam_opt->lamSpeed = myGetIntOption( "LamSpeed", num_options, options, ppd, 4 );
    
    if ( (lam_opt->lamSpeed<2) && (lam_opt->lamSpeed>18) ) lam_opt->lamSpeed=4; 
    info("LamSpeed : %d", lam_opt->lamSpeed);
    
    lam_opt->lamTemp = myGetIntOption( "Lamtemp", num_options, options, ppd, 155 );
    
    if ( (lam_opt->lamTemp<100) && (lam_opt->lamTemp>180) ) lam_opt->lamTemp=155;           
    info("Lamtemp : %d", lam_opt->lamTemp);
    
    myGetStrOption( lam_opt->lamDesc, "LamDescription", num_options, options, ppd, "Custom" ); 
    
    lam_opt->lamDesc[16] = 0; // on place un caractere de fin dans tous les cas au bout du tableau (voir strncpy)
    info("LamDescription : %s", lam_opt->lamDesc);
    
    return 1;
}

// ----------------------------------------------------
// int ProcessPassthroughOption( int num_options, cups_option_t *options, ppd_file_t *ppd )
//
// @brief Process Passthrough options
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ----------------------------------------------------
int ProcessPassthroughOption( int num_options, cups_option_t *options, ppd_file_t *ppd )
{
    const char      *val;                   /* CUPS option value */ 
    char WtData[MAX_PASSTHROUGH_SZ];
    int  i = 0, j = 0;

    if ((val = cupsGetOption("RawData", num_options, options)) != NULL) 
        strcpy(WtData,val);
    else
        if ((val = cupsGetOption("passthrough", num_options, options)) != NULL) 
            strcpy(WtData,val);
        else 
            WtData[0] = 0x00;
    
    j = 0;  i = 0;  
    info("*** Passthrough : %s", &WtData[j]);
    if (WtData[0] != 0x00) output("\033Wcb;k\015");   // K panel printing Init.

    while (WtData[i] != 0x00 && i < MAX_PASSTHROUGH_SZ) 
    {
        if((WtData[i] == '|') && i) 
        {
            WtData[i] = 0x00;
            output("\033%s\015",&WtData[j]);
            info("Passthrough : %s", &WtData[j]);
            j=i+1;        
        }
        i++;
    }
	if ((i!=j) && (i < MAX_PASSTHROUGH_SZ))  
	{	
		// WtData[i] = 0;
		debug("Passthrough : %s", &WtData[j]);
		output("\033%s\015", &WtData[j]);
	}
    return 1; 

}

// ----------------------------------------------------
// int GetMagOption( LaminatorOption_t *lam_opt, int num_options, cups_option_t *options, ppd_file_t *ppd )
//
// @brief parse Magnetic encoding options
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ----------------------------------------------------
int GetMagOption( MagInfo_t *mag, int num_options, cups_option_t *options, ppd_file_t *ppd )
{
    const char      *val;                   /* CUPS option value */ 
    char            tmp[10];

    myGetStrOption(mag->Coercivity, "Coercivity", num_options, options, ppd, "None" ); 
    debug("Coercivity : %s", mag->Coercivity);
    
    if(strcasecmp(mag->Coercivity,"NONE") || !(strcasecmp(EjectCard, "ONLYENCO")) ) 
    {
        mag->Track[0].type = myGetIntOption( "Track1", num_options, options, ppd, 1 ); 
        mag->Track[1].type = myGetIntOption( "Track2", num_options, options, ppd, 2 ); 
        mag->Track[2].type = myGetIntOption( "Track3", num_options, options, ppd, 3 ); 
        
        debug("Track\t 1: %d\t  2: %d\t  3: %d", mag->Track[0].type,
              mag->Track[1].type,
              mag->Track[2].type );
        
        mag->Track[0].density = myGetIntOption( "Density1", num_options, options, ppd, 0 );         
        mag->Track[1].density = myGetIntOption( "Density2", num_options, options, ppd, 0 );         
        mag->Track[2].density = myGetIntOption( "Density3", num_options, options, ppd, 0 );         
        
        debug("Density\t 1: %d\t  2: %d\t  3: %d", mag->Track[0].density,
              mag->Track[1].density,
              mag->Track[2].density );
        
        myGetStrOption( tmp, "KeepCard", num_options, options, ppd, "True" );           
        if ( (tmp[0] == 'F') || (tmp[0] == 'f')) 
            mag->KeepCard = 0;
        else
            mag->KeepCard = 1;
        debug("KeepCard : %d", mag->KeepCard);

        if ((val = cupsGetOption("Track1Data", num_options, options)) != NULL) 
        {
            sprintf( mag->Track[0].data, "\033Dm;1;%s\015", val );
            info("%s", mag->Track[0].data );
        }
        else
            mag->Track[0].data[0] = 0;
        
        if ((val = cupsGetOption("Track2Data", num_options, options)) != NULL) 
        {
            sprintf( mag->Track[1].data, "\033Dm;2;%s\015", val );
            info("%s", mag->Track[1].data );
        }
        else
            mag->Track[1].data[0] = 0;
        
        if ((val = cupsGetOption("Track3Data", num_options, options)) != NULL) 
        {
            sprintf( mag->Track[2].data, "\033Dm;3;%s\015", val );
            info("%s", mag->Track[2].data );
        }
        else
            mag->Track[2].data[0] = 0;
                    info("Set Magnetic encoder");
        
        return SendMagEncodingHeader( mag );
    }
    return 1;
}

// ----------------------------------------------------
// int GetPageSetupOption( int num_options, cups_option_t *options, ppd_file_t *ppd )
//
// @brief parse page setup options
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ---------------------------------------------------- 
int GetPageSetupOption( int num_options, cups_option_t *options, ppd_file_t *ppd )
{
    const char      *val;                   /* CUPS option value */ 

    if ((val = cupsGetOption("orientation", num_options, options)) != NULL) 
    {
        if(!strcasecmp(val,"PORTRAIT")) 
            cOrientation = PORTRAIT;
        else 
            cOrientation = LANDSCAPE;
        debug("cOrientation : %d", cOrientation);
    }
    
    if ((val = cupsGetOption("Sides", num_options, options)) != NULL) 
    {
        if(!strcasecmp(val,"ONE-SIDED")) 
            cSides = 0;
        else 
            cSides = 1;
        debug("cSides : %d", cSides);
    }
    
    if ((val = cupsGetOption("page-left", num_options, options)) != NULL) 
    {
        lMargin = atoi(val);
        debug("lMargin : %d", lMargin);
    }
    
    if ((val = cupsGetOption("page-top", num_options, options)) != NULL)
    {
        tMargin = atoi(val);
        debug("tMargin : %d", lMargin);
    }
    if ((val = cupsGetOption("page-right", num_options, options)) != NULL) 
    {
        rMargin = atoi(val);
        debug("rMargin : %d", lMargin);
    }
    if ((val = cupsGetOption("page-bottom", num_options, options)) != NULL)
    {
        bMargin = atoi(val);
        debug("bMargin : %d", lMargin);
    }
    return 1;
}

// ----------------------------------------------------
// int GetEvolisOption( int num_options, cups_option_t *options, ppd_file_t *ppd )
//
// @brief parse evolis printer options
// @author fhl
// @return  1 -  ok 
//          0 - nok
// ---------------------------------------------------- 
int GetEvolisOption( int num_options, cups_option_t *options, ppd_file_t *ppd )
{
    const char      *val;                   /* CUPS option value */ 

    myGetStrOption( EjectCard, "EjectCard", num_options, options, ppd, "None" );
    debug("EjectCard : %s", EjectCard);
    
    myGetStrOption( ColorSmooth, "ColorSmooth", num_options, options, ppd, "VeryHigh" );
    debug("ColorSmooth : %s", ColorSmooth); // todo fhl : remove comment
    
    Brightness[RECTO] = myGetIntOption( "Brightness", num_options, options, ppd, 10 );
    debug("Brightness (recto) : %d", Brightness[RECTO]);
    
    Brightness[VERSO] = myGetIntOption( "BrightnessV", num_options, options, ppd, Brightness[RECTO] );
    debug("Brightness (verso) : %d", Brightness[VERSO]);
    
    Contrast[RECTO]  = myGetIntOption( "Contrast", num_options, options, ppd, 10 );
    debug("Contrast (recto) : %d", Contrast[RECTO] );
    
    Contrast[VERSO] = myGetIntOption( "ContrastV", num_options, options, ppd, Contrast[RECTO] );
    debug("Contrast (verso)  : %d", Contrast[VERSO]);

    SensibilityK[RECTO]  = myGetIntOption( "SensibilityK", num_options, options, ppd, 10 );
    debug("SensibilityK (recto) : %d", SensibilityK[RECTO] );
    
    SensibilityK[VERSO] = myGetIntOption( "SensibilityKV", num_options, options, ppd, SensibilityK[RECTO] );
    debug("SensibilityK (verso)  : %d", SensibilityK[VERSO]);

    SensibilityO[RECTO]  = myGetIntOption( "SensibilityO", num_options, options, ppd, 10 );
    debug("SensibilityO (recto) : %d", SensibilityO[RECTO] );
    
    SensibilityO[VERSO] = myGetIntOption( "SensibilityOV", num_options, options, ppd, SensibilityO[RECTO] );
    debug("SensibilityO (verso)  : %d", SensibilityO[VERSO]);
    
    myGetStrOption( OverlayPannel, "OverlayPannel", num_options, options, ppd, "FO" );
    debug("OverlayPannel : %s", OverlayPannel);
    
    myGetStrOption( OverlayBackPannel, "OverlayBackPannel", num_options, options, ppd, "FO" );
    debug("OverlayBackPannel : %s", OverlayBackPannel);
    
    overlayRegion[RECTO][0] = myGetIntOption( "TopCover", num_options, options, ppd, 0 );
    debug("TB: %d",overlayRegion[RECTO][0]);  
    
    overlayRegion[RECTO][1] = myGetIntOption( "TopLeft", num_options, options, ppd, 0 );
    debug("LB: %d",overlayRegion[RECTO][1]);
    
    overlayRegion[RECTO][2] = myGetIntOption( "BottomCover", num_options, options, ppd, 0 );
    debug("BB: %d",overlayRegion[RECTO][2]);
    
    overlayRegion[RECTO][3] = myGetIntOption( "BottomRight", num_options, options, ppd, 0 );
    debug("RB: %d",overlayRegion[RECTO][3]);
    
    overlayRegion[RECTO][4] = myGetIntOption( "TopWCover", num_options, options, ppd, 0 );
    debug("TW: %d",overlayRegion[RECTO][4]);
    
    overlayRegion[RECTO][5] = myGetIntOption( "TopWLeft", num_options, options, ppd, 0 );
    debug("LW: %d",overlayRegion[RECTO][5]);
    
    overlayRegion[RECTO][6] = myGetIntOption( "BottomWCover", num_options, options, ppd, 0 );
    debug("BW: %d",overlayRegion[RECTO][6]);
    
    overlayRegion[RECTO][7] = myGetIntOption( "BottomWRight", num_options, options, ppd, 0 );
    debug("RW: %d",overlayRegion[RECTO][7]);

    overlayRegion[VERSO][0] = myGetIntOption( "TopCoverV", num_options, options, ppd, 0 );
    debug("TB: %d",overlayRegion[VERSO][0]);  
    
    overlayRegion[VERSO][1] = myGetIntOption( "TopLeftV", num_options, options, ppd, 0 );
    debug("LB: %d",overlayRegion[VERSO][1]);
    
    overlayRegion[VERSO][2] = myGetIntOption( "BottomCoverV", num_options, options, ppd, 0 );
    debug("BB: %d",overlayRegion[VERSO][2]);
    
    overlayRegion[VERSO][3] = myGetIntOption( "BottomRightV", num_options, options, ppd, 0 );
    debug("RB: %d",overlayRegion[VERSO][3]);
    
    overlayRegion[VERSO][4] = myGetIntOption( "TopWCoverV", num_options, options, ppd, 0 );
    debug("TW: %d",overlayRegion[VERSO][4]);
    
    overlayRegion[VERSO][5] = myGetIntOption( "TopWLeftV", num_options, options, ppd, 0 );
    debug("LW: %d",overlayRegion[VERSO][5]);
    
    overlayRegion[VERSO][6] = myGetIntOption( "BottomWCoverV", num_options, options, ppd, 0 );
    debug("BW: %d",overlayRegion[VERSO][6]);
    
    overlayRegion[VERSO][7] = myGetIntOption( "BottomWRightV", num_options, options, ppd, 0 );
    debug("RW: %d",overlayRegion[VERSO][7]);
    
    BlackIn[RECTO] = myGetCharOption( "BlackIn", num_options, options, ppd, 'N' );
    info/*debug*/("BlackIn (recto) : %c", BlackIn[RECTO]);
    
    BlackIn[VERSO] = myGetCharOption(  "BlackInV", num_options, options, ppd, 'N' );
    info/*debug*/("BlackIn (verso) : %c", BlackIn[VERSO]);
    
    TreatementK[RECTO] = myGetCharOption( "TreatementK", num_options, options, ppd, 'G' );
    debug("TreatementK (recto) : %c", TreatementK[RECTO]);
    
    TreatementK[VERSO] = myGetCharOption(  "TreatementKV", num_options, options, ppd, 'G' );
    debug("TreatementK (verso) : %c", TreatementK[VERSO]);
    
    QualityK = myGetIntOption( "QualityK", num_options, options, ppd, 0 );
    debug("QualityK : %d", QualityK);
    
    // fhl - option retiree des ppds - 26/06/08
    //          concerne la detection auto du debut d'impression quand ymckos
    
    DetectAuto = myGetIntOption( "DetectAuto", num_options, options, ppd, 1);   
    debug("DetectAuto : %d", DetectAuto);
    
    StartDist = myGetIntOption( "StartDist", num_options, options, ppd, 0) * 12;    
    debug("StartDist : %d", StartDist);
    
    myGetStrOption( Soften, "Soften", num_options, options, ppd, "FALSE" ); 
    // fhl - 30/06/08 - option retiree de l'interface ppd 
    //                  FALSE active l'option (augmentation de la qualite de l'image)
    debug("Soften : %s", Soften);
    
    return 1;
}

// ----------------------------------------------------
// int      SetOverlayRegion        ( int overlayRegion[8] )
//
// @brief swap overlay region description 
// @author fhl
// @return  1 -  ok 
//          0 -  nok
// ----------------------------------------------------
int     SetOverlayRegion        ( int region[8] )
{
    TB = region[0];
    LB = region[1];
    BB = region[2];
    RB = region[3];
    TW = region[4];
    LW = region[5];
    BW = region[6];
    RW = region[7];
    
    return 1;
}
/* End of rastertoevolis.c,v 2.0*/

